function Q = Simpson(x , y)
%SIMPSON Numerical evaluation of integral by Simpson's 1/3 rule.
%
%     SIMPSON(X,Y) numerically evaluates the integral of the
%     vector of function values Y with respect to X by
%     Simpson's 1/3 rule. X is the vector of equally spaced
%     independent variable. Length of Y has to be odd (even
%     number of intervals). If length of Y is even, the function
%     calculates the integral for [LENGTH(Y)-1] points by 
%     Simpson's 1/3 rule and adds to it the value of the 
%     integral for the last interval by trapezoidal rule.
%
%     See also TRAPZ , QUAD , QUAD8, GAUSSLEGENDRE

% (c) N. Mostoufi & A. Constantinides
% January 1, 1999

points = length(x);
if length(y) ~= points
   error('x and y are not of the same length')
   break
end

dx = diff(x);
if max(dx)-min(dx) > min(abs(x))/1000
   error('X is not equally spaced.')
   break
end
h = dx(1);

if mod(points,2) == 0
   warning('Odd number of intervals; Trapezoidal rule will be used for the last interval.')
   n = points - 1;
else
   n = points;
end

% Integration
y1 = y(2 : 2 : n - 1);
y2 = y(3 : 2 : n - 2);
Q = (y(1) + 4 * sum(y1) + 2 * sum(y2) + y(n)) * h /3;

if n ~= points
   Q = Q + (y(points) + y(n)) * h / 2;
end
