function [x,y] = AdamsMoulton(ODEfile,xi,xf,h,yi,varargin)
% ADAMSMOULTON Solves a set of ordinary differential equations by
%   the Adams-Moulton method.
%
%   [X,Y]=ADAMSMOULTON('F',XI,XF,H,YI) solves a set of ordinary
%   differential equations by the Adams-Moulton method, from XI to
%   XF.  The equations are given in the M-file F.M.   H is the
%   length of interval and YI is the vector of initial values of
%   the dependent variable at XI.
%
%   [X,Y]=ADAMSMOULTON('F',XI,XF,H,YI,P1,P2,...) allows for
%   additional arguments which are passed to the function
%   F(X,P1,P2,...).
%
%  See also ODE23, ODE45, ODE113, ODE15S, ODE23S, EULER, MEULER,
%           RK, ADAMS

% (c) N. Mostoufi & A. Constantinides
% January 1, 1999

% Initialization
if isempty(h) | h == 0
   h = (xf - xi)/99;
end

yi = (yi(:).')';		% Make sure it's a column vector

x = [xi:h:xf]';		% Vector of x values
if x(end) ~= xf
   x(end+1) = xf;
end
d = diff(x);			% Vector of x-increments

% Starting values
[a,b] = RK(ODEfile,x(1),x(4),h,yi,4,varargin{:});
y(:,1:4) = b;
for i = 1:4
   f(:,i) = feval(ODEfile,x(i),y(:,i),varargin{:});
end

% Solution
for i = 4:length(x)-1
   % Predictor
   y(:,i+1) = y(:,i) + d(i)/24 * (55*f(:,i) - 59*f(:,i-1) ...
      + 37*f(:,i-2) - 9*f(:,i-3));
   f(:,i+1) = feval(ODEfile,x(i+1),y(:,i+1),varargin{:});
   % Corrector
   y(:,i+1) = y(:,i) + d(i)/24 * (9*f(:,i+1) + 19*f(:,i) ...
      - 5*f(:,i-1) + f(:,i-2));
   f(:,i+1) = feval(ODEfile,x(i+1),y(:,i+1),varargin{:});
end
